import io
import json
import os

import click
from jinja2 import Template

from booknot.crawler.crawler import Crawler


class CaptureApplication:

    def __init__(self, crawler: Crawler, directory, url):
        self.crawler = crawler
        self.directory = os.path.realpath(directory)
        self.url = url

    def run(self):
        booknot_path = os.path.join(self.directory, '.booknot')
        template_booknot_path = os.path.join(self.directory, '.booknot', 'index.rst.j2')

        if not os.path.isdir(booknot_path):
            raise click.ClickException(f'you have to initialise booknot repository wiht booknot init :  {booknot_path}')

        if not os.path.isfile(template_booknot_path):
            raise click.ClickException(f'template for booknot does not exists {template_booknot_path}')

        metadata = self.crawler.fetch(self.url)

        booknot_note_path = os.path.join(self.directory, metadata.sanitize_dir())
        os.makedirs(booknot_note_path)
        os.makedirs(os.path.join(booknot_note_path, 'static'))
        with io.open(os.path.join(booknot_note_path, 'manifest.json'), 'w') as filep:
            json.dump(metadata.tomanifest(), filep, indent=4)

        with io.open(template_booknot_path) as filep:
            template_index = filep.read()
            template = Template(template_index)
            template_render = template.render(metadata.tocapture())
            with io.open(os.path.join(booknot_note_path, 'index.rst'), 'w') as filep:
                filep.write(template_render)



#!/usr/bin/python
# coding=utf-8

from __future__ import print_function

import os

import click
import requests
from bs4 import BeautifulSoup

from booknot.application.init_application import InitApplication
from booknot.application.capture_application import CaptureApplication
from booknot.booknot_storage.local_booknot_storage import LocalBooknotStorage
from booknot.crawler.http_crawler import HttpCrawler


@click.group()
def cli():
    pass


@click.command('init')
def init():
    workdir = os.getcwd()
    booknot_storage = LocalBooknotStorage(workdir)
    init_application = InitApplication(booknot_storage)

    # init booknot root
    init_application.run()


@click.command('capture')
@click.argument('url')
def capture(url):
    workdir = os.getcwd()
    session = requests.Session()
    crawler = HttpCrawler(session)
    capture_application = CaptureApplication(crawler, workdir, url)
    capture_application.run()


cli.add_command(init)
cli.add_command(capture)

if __name__ == '__main__':
    cli()



import os
import shutil

import click
import plumbum

from booknot.booknot_storage.booknot_storage import BooknotStorage
from booknot.resources import RESOURCES_DIR


class LocalBooknotStorage(BooknotStorage):

    def __init__(self, directory):
        self.directory = directory
        self.booknot_meta_directory = os.path.join(self.directory, '.booknot')
        self.booknot_toctree = os.path.join(self.directory, 'index.rst')

    def exists(self) -> bool:
        return os.path.isdir(self.booknot_meta_directory)

    def init_store(self):
        try:
            if not os.path.isdir(self.booknot_meta_directory):
                shutil.copytree(os.path.join(RESOURCES_DIR, 'booknot_root'), self.booknot_meta_directory)

            if not os.path.isfile(self.booknot_toctree):
                shutil.copy(os.path.join(RESOURCES_DIR, 'toctree.rst'), self.booknot_toctree)
        except Exception as exception:
            if os.path.isdir(self.booknot_meta_directory):
                shutil.rmtree(self.booknot_meta_directory)

            raise click.ClickException(f'invalid exception - {exception}')

    def create_sphinx(self, project, author) -> None:
        sphinx = plumbum.local['sphinx-quickstart']
        sphinx(['-q', '-p', project, '-a', author, self.directory])
        os.remove('index.rst')

    def is_sphinx_present(self):
        is_sphinx = False
        path = os.path.normpath(self.directory)
        directories_part = path.split(os.sep)
        while True:
            if len(directories_part) == 0:
                break

            directories_part.pop()
            directory = os.sep.join(directories_part)
            if os.path.isfile(os.path.join(directory, 'conf.py')):
                is_sphinx = True
                break

        return is_sphinx



import os
import unittest
from datetime import datetime
from unittest.mock import Mock

from booknot.application.capture_application import CaptureApplication
from booknot.crawler.metadata import Metadata
from booknot_tests.fixtures import clone_fixture


class TestCatpureApplication(unittest.TestCase):
    def setUp(self):
        self.crawler_mock = Mock()
        self._tested = CaptureApplication(self.crawler_mock, '', 'http://any')

    def test_run_should_create_a_directory_for_the_note(self):
        with clone_fixture('empty_booknot') as directory:
            # Assign
            self._tested.directory = directory
            self.crawler_mock.fetch = Mock(return_value=Metadata(url="", title="my title", description="", capture_date=datetime(2020, 1, 1)))

            # Acts
            result = self._tested.run()

            # Assert
            note_directory = os.path.join(directory, '20200101__my_title')

            self.assertTrue(os.path.isdir(note_directory), f'note directory does not exists in {note_directory}')

    def test_run_should_create_the_manifest_of_the_note(self):
        with clone_fixture('empty_booknot') as directory:
            # Assign
            self._tested.directory = directory
            self.crawler_mock.fetch = Mock(return_value=Metadata(url="", title="my title", description="", capture_date=datetime(2020, 1, 1)))

            # Acts
            result = self._tested.run()

            # Assert
            note_manifest = os.path.join(directory, '20200101__my_title', 'manifest.json')

            self.assertTrue(os.path.isfile(note_manifest), f'note manifest does not exists in {note_manifest}')

    def test_run_should_create_the_index_of_the_note(self):
        with clone_fixture('empty_booknot') as directory:
            # Assign
            self._tested.directory = directory
            self.crawler_mock.fetch = Mock(return_value=Metadata(url="", title="my title", description="", capture_date=datetime(2020, 1, 1)))

            # Acts
            result = self._tested.run()

            # Assert
            note_index = os.path.join(directory, '20200101__my_title', 'index.rst')

            self.assertTrue(os.path.isfile(note_index), f'note manifest does not exists in {note_index}')

if __name__ == '__main__':
    unittest.main()



from datetime import datetime

import attr


@attr.s
class Metadata:
    url: str = attr.ib()
    title: str = attr.ib()
    description: str = attr.ib()
    capture_date: datetime = attr.ib(default=datetime.min)

    def sanitize_dir(self) -> str:
        sanitize_title = self.title.lower().replace(' ', '_').replace('|', '_').replace(':', '_')\
            .replace('/', '_').replace('"', '')
        sanitize_date = self.capture_date.strftime('%Y%m%d')

        return f'{sanitize_date}__{sanitize_title}'

    def tomanifest(self) -> dict:
        serialized = attr.asdict(self)
        serialized["capture_date"] = self.capture_date.isoformat()
        return serialized

    def tocapture(self) -> dict:
        serialized = attr.asdict(self)
        serialized["capture_date"] = self.capture_date.strftime('%d/%m/%Y')
        return serialized



import os
import unittest

from booknot.booknot_storage.local_booknot_storage import LocalBooknotStorage
from booknot_tests.fixtures import clone_fixture


class TestLocalBooknotStorage(unittest.TestCase):
    def setUp(self):
        pass

    def test_init_should_create_dot_booknote_directory(self):
        # Assign
        with clone_fixture('empty_directory') as directory:
            booknot_storage = LocalBooknotStorage(directory)

            # Acts
            booknot_storage.init_store()

            self.assertTrue(os.path.isdir(os.path.join(directory, '.booknot')))

    def test_init_should_populate_dot_booknote_directory(self):
        # Assign
        with clone_fixture('empty_directory') as directory:
            booknot_storage = LocalBooknotStorage(directory)

            # Acts
            booknot_storage.init_store()

            self.assertTrue(os.path.isdir(os.path.join(directory, '.booknot')))
            self.assertTrue(os.path.isfile(os.path.join(directory, '.booknot', 'index.rst.j2')))

    def test_is_sphinx_present_should_check_conf_py_is_present_in_parent(self):
        # Assign
        with clone_fixture('sphinx') as directory:
            working_dir = os.path.join(directory, 'reading_notes')
            booknot_storage = LocalBooknotStorage(working_dir)

            # Acts
            is_present = booknot_storage.is_sphinx_present()

            self.assertTrue(is_present)

    def test_is_sphinx_present_should_return_false_when_sphinx_is_not_present(self):
        # Assign
        with clone_fixture('empty_directory') as directory:
            working_dir = os.path.join(directory)
            booknot_storage = LocalBooknotStorage(working_dir)

            # Acts
            is_present = booknot_storage.is_sphinx_present()

            self.assertFalse(is_present)

if __name__ == '__main__':
    unittest.main()



from abc import ABCMeta, abstractmethod

from booknot.crawler.metadata import Metadata


class Crawler:
    __metaclass__ = ABCMeta

    @abstractmethod
    def fetch(self, url: str) -> Metadata:
        raise NotImplementedError()



import os
import shutil
import tempfile

from decorator import contextmanager

SCRIPT_DIRECTORY_PATH = os.path.dirname(os.path.realpath(__file__))

@contextmanager
def clone_fixture(fixture_name, working_dir=None):
  tmp_prefix = '{0}_{1}'.format(fixture_name, '_')
  working_directory = tempfile.mktemp(prefix=tmp_prefix) if not working_dir else working_dir
  template_working_directory = os.path.join(SCRIPT_DIRECTORY_PATH, fixture_name)

  if not os.path.isdir(template_working_directory):
    fixtures_list = [d for d in os.listdir(SCRIPT_DIRECTORY_PATH) if
                        os.path.isdir(os.path.join(SCRIPT_DIRECTORY_PATH, d))]
    raise Exception('the fixture {0} does not exists in fixtures : {1}'.format(fixture_name, fixtures_list))

  shutil.copytree(template_working_directory, working_directory)
  yield working_directory

  shutil.rmtree(working_directory)


import unittest
from datetime import datetime

from booknot.crawler.metadata import Metadata


class TestMetadata(unittest.TestCase):
    def setUp(self):
        pass

    def test_sanitize_dir_should_remove_uppercase_and_space(self):
        # Assign
        metadata = Metadata(url="", title="Yolo my title", description="", capture_date=datetime(2020, 1, 1))

        # Acts
        directory_name = metadata.sanitize_dir()

        # Assert
        self.assertEqual("20200101__yolo_my_title", directory_name)

    def test_sanitize_dir_should_remove_pipe(self):
        # Assign
        metadata = Metadata(url="", title="Yolo my | title", description="", capture_date=datetime(2020, 1, 1))

        # Acts
        directory_name = metadata.sanitize_dir()

        # Assert
        self.assertEqual("20200101__yolo_my___title", directory_name)

    def test_tomanifest_should_convert_date_to_iso_format(self):
        # Assign
        metadata = Metadata(url="", title="Yolo my | title", description="", capture_date=datetime(2020, 1, 1))

        # Acts
        manifest = metadata.tomanifest()

        # Assert
        self.assertEqual("2020-01-01T00:00:00", manifest['capture_date'])

    def test_tocapture_should_convert_date_to_readable_format(self):
        # Assign
        metadata = Metadata(url="", title="Yolo my | title", description="", capture_date=datetime(2020, 1, 1))

        # Acts
        manifest = metadata.tocapture()

        # Assert
        self.assertEqual("01/01/2020", manifest['capture_date'])

if __name__ == '__main__':
    unittest.main()



import click
import inquirer

from booknot.booknot_storage.booknot_storage import BooknotStorage

class InitApplication:

    def __init__(self, booknot_storage: BooknotStorage):
        self.booknot_storage = booknot_storage

    def run(self):

        if self.booknot_storage.exists():
            raise click.ClickException('.booknot already exists')

        if not self.booknot_storage.is_sphinx_present():
            questions = [
                inquirer.Confirm('init sphinx',
                                 message='This will create a workspace for sphinx, do you want to continue ?',
                                 default=False),
            ]

            answers = inquirer.prompt(questions)
            if answers['init sphinx']:
                questions = [
                    inquirer.Text('project',
                                     message='What the name of the project of this booknot',
                                     default='Booknot'),
                    inquirer.Text('author',
                                     message='What the name of the author',
                                     default='me'),
                ]

                answers = inquirer.prompt(questions)
                self.booknot_storage.create_sphinx(answers['project'], answers['author'])

                click.echo("to render the documentation, use make html")
                click.echo("to open the render, use open _build/html/index.html")
                self.booknot_storage.init_store()
        else:
            self.booknot_storage.init_store()
            click.echo("you can use : booknot capture https://...")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup
from setuptools import find_packages

setup(
    name='booknot',
    version='0.0.2',
    packages=find_packages(exclude=["*_tests"]),
    license='MIT license',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    include_package_data=True,
    package_data={
        'configuration': ['booknot/resources/*'],
    },
    entry_points = {
        'console_scripts': [
            'booknot = booknot.cli:cli',
        ],
    },
    install_requires = [
        'attrs',
        'beautifulsoup4',
        'click',
        'decorator',
        'inquirer',
        'jinja2',
        'plumbum',
        'requests',
        'sphinx'
    ],
    extras_require={
        'dev': [
            'pylint',
            'coverage',
            'tox',
            'twine'
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Environment :: Console"
    ]
)



from abc import ABCMeta, abstractmethod


class BooknotStorage:
    __metaclass__ = ABCMeta

    @abstractmethod
    def exists(self) -> bool:
        raise NotImplementedError()

    @abstractmethod
    def init_store(self) -> None:
        raise NotImplementedError()

    @abstractmethod
    def create_sphinx(self, project, author) -> None:
        raise NotImplementedError()

    @abstractmethod
    def is_sphinx_present(self) -> bool:
        raise NotImplementedError()



from datetime import datetime

import requests
from bs4 import BeautifulSoup

from booknot.crawler.crawler import Crawler
from booknot.crawler.metadata import Metadata


class HttpCrawler(Crawler):

    def __init__(self, session: requests.Session):
        self.session = session

    def fetch(self, url: str) -> Metadata:
        r = self.session.get(url)
        content = r.content
        soup = BeautifulSoup(content, 'html.parser')
        title = soup.head.title.string
        description = soup.head.find('meta', property="og:description")["content"]
        now = datetime.now()

        return Metadata(url=url, title=title, description=description, capture_date=now)
